@@ -8,7 +8,7 @@ from rest_framework import exceptions
 
 from caluma.caluma_data_source.data_source_handlers import get_data_sources
 
-from . import jexl, structure
+from . import jexl, models, structure
 from .format_validators import get_format_validators
 from .models import DynamicOption, Question
 
@@ -176,6 +176,11 @@ class AnswerValidator:
     def _validate_question_file(self, question, value, **kwargs):
         pass
 
+    def _validate_question_calculated_float(self, question, value, **kwargs):
+        raise CustomValidationError(
+            "Cannot save read-only answer of of type calculated float."
+        )
+
     def validate(
         self,
         *,
@@ -265,7 +270,7 @@ class DocumentValidator:
         for field in validation_context["structure"].children():
             question = field.question
             try:
-                is_hidden = q_jexl.is_hidden(question)
+                is_hidden = q_jexl.is_hidden(field)
 
                 if is_hidden:
                     # no need to descend further
@@ -329,8 +334,8 @@ class DocumentValidator:
                 # in another row, but would still be hidden in the local row, if this is a
                 # table question context.  Thus, in this case we need to re-evaluate it's
                 # hiddenness. Luckily, the JEXL evaluator caches those values (locally).
-                with q_jexl.use_question_context(question.pk):
-                    if q_jexl.is_hidden(question):
+                with q_jexl.use_field_context(field):
+                    if q_jexl.is_hidden(field):
                         continue
 
                 is_required = q_jexl.is_required(field)
@@ -396,9 +401,31 @@ class QuestionValidator:
         if data_source not in data_sources:
             raise exceptions.ValidationError(f'Invalid data_source: "{data_source}"')
 
+    @staticmethod
+    def _validate_calc_dependencies(data):
+        expr = data.get("calc_expression")
+        if not expr:
+            return
+
+        question_jexl = jexl.QuestionJexl()
+        deps = set(question_jexl.extract_referenced_questions(expr))
+        illegal_deps = ", ".join(
+            models.Question.objects.filter(
+                pk__in=deps, type=models.Question.TYPE_CALCULATED_FLOAT
+            ).values_list("slug", flat=True)
+        )
+
+        if illegal_deps:
+            raise exceptions.ValidationError(
+                f"Calc expression references other calculated question: {illegal_deps}"
+            )
+
     def validate(self, data):
-        if data["type"] in ["text", "textarea"]:
+        if data["type"] in [models.Question.TYPE_TEXT, models.Question.TYPE_TEXTAREA]:
             self._validate_format_validators(data)
+        elif data["type"] == models.Question.TYPE_CALCULATED_FLOAT:
+            self._validate_calc_dependencies(data)
+
         if "dataSource" in data:
             self._validate_data_source(data["dataSource"])
 
